{-# language CPP #-}
-- | = Name
--
-- VK_EXT_display_control - device extension
--
-- == VK_EXT_display_control
--
-- [__Name String__]
--     @VK_EXT_display_control@
--
-- [__Extension Type__]
--     Device extension
--
-- [__Registered Extension Number__]
--     92
--
-- [__Revision__]
--     1
--
-- [__Extension and Version Dependencies__]
--
--     -   Requires Vulkan 1.0
--
--     -   Requires @VK_EXT_display_surface_counter@
--
--     -   Requires @VK_KHR_swapchain@
--
-- [__Contact__]
--
--     -   James Jones
--         <https://github.com/KhronosGroup/Vulkan-Docs/issues/new?title=VK_EXT_display_control:%20&body=@cubanismo%20 >
--
-- == Other Extension Metadata
--
-- [__Last Modified Date__]
--     2016-12-13
--
-- [__IP Status__]
--     No known IP claims.
--
-- [__Contributors__]
--
--     -   Pierre Boudier, NVIDIA
--
--     -   James Jones, NVIDIA
--
--     -   Damien Leone, NVIDIA
--
--     -   Pierre-Loup Griffais, Valve
--
--     -   Daniel Vetter, Intel
--
-- == Description
--
-- This extension defines a set of utility functions for use with the
-- @VK_KHR_display@ and @VK_KHR_display_swapchain@ extensions.
--
-- == New Commands
--
-- -   'displayPowerControlEXT'
--
-- -   'getSwapchainCounterEXT'
--
-- -   'registerDeviceEventEXT'
--
-- -   'registerDisplayEventEXT'
--
-- == New Structures
--
-- -   'DeviceEventInfoEXT'
--
-- -   'DisplayEventInfoEXT'
--
-- -   'DisplayPowerInfoEXT'
--
-- -   Extending
--     'Vulkan.Extensions.VK_KHR_swapchain.SwapchainCreateInfoKHR':
--
--     -   'SwapchainCounterCreateInfoEXT'
--
-- == New Enums
--
-- -   'DeviceEventTypeEXT'
--
-- -   'DisplayEventTypeEXT'
--
-- -   'DisplayPowerStateEXT'
--
-- == New Enum Constants
--
-- -   'EXT_DISPLAY_CONTROL_EXTENSION_NAME'
--
-- -   'EXT_DISPLAY_CONTROL_SPEC_VERSION'
--
-- -   Extending 'Vulkan.Core10.Enums.StructureType.StructureType':
--
--     -   'Vulkan.Core10.Enums.StructureType.STRUCTURE_TYPE_DEVICE_EVENT_INFO_EXT'
--
--     -   'Vulkan.Core10.Enums.StructureType.STRUCTURE_TYPE_DISPLAY_EVENT_INFO_EXT'
--
--     -   'Vulkan.Core10.Enums.StructureType.STRUCTURE_TYPE_DISPLAY_POWER_INFO_EXT'
--
--     -   'Vulkan.Core10.Enums.StructureType.STRUCTURE_TYPE_SWAPCHAIN_COUNTER_CREATE_INFO_EXT'
--
-- == Issues
--
-- 1) Should this extension add an explicit “WaitForVsync” API or a fence
-- signaled at vsync that the application can wait on?
--
-- __RESOLVED__: A fence. A separate API could later be provided that
-- allows exporting the fence to a native object that could be inserted
-- into standard run loops on POSIX and Windows systems.
--
-- 2) Should callbacks be added for a vsync event, or in general to monitor
-- events in Vulkan?
--
-- __RESOLVED__: No, fences should be used. Some events are generated by
-- interrupts which are managed in the kernel. In order to use a callback
-- provided by the application, drivers would need to have the userspace
-- driver spawn threads that would wait on the kernel event, and hence the
-- callbacks could be difficult for the application to synchronize with its
-- other work given they would arrive on a foreign thread.
--
-- 3) Should vblank or scanline events be exposed?
--
-- __RESOLVED__: Vblank events. Scanline events could be added by a
-- separate extension, but the latency of processing an interrupt and
-- waking up a userspace event is high enough that the accuracy of a
-- scanline event would be rather low. Further, per-scanline interrupts are
-- not supported by all hardware.
--
-- == Version History
--
-- -   Revision 1, 2016-12-13 (James Jones)
--
--     -   Initial draft
--
-- = See Also
--
-- 'DeviceEventInfoEXT', 'DeviceEventTypeEXT', 'DisplayEventInfoEXT',
-- 'DisplayEventTypeEXT', 'DisplayPowerInfoEXT', 'DisplayPowerStateEXT',
-- 'SwapchainCounterCreateInfoEXT', 'displayPowerControlEXT',
-- 'getSwapchainCounterEXT', 'registerDeviceEventEXT',
-- 'registerDisplayEventEXT'
--
-- = Document Notes
--
-- For more information, see the
-- <https://www.khronos.org/registry/vulkan/specs/1.2-extensions/html/vkspec.html#VK_EXT_display_control Vulkan Specification>
--
-- This page is a generated document. Fixes and changes should be made to
-- the generator scripts, not directly.
module Vulkan.Extensions.VK_EXT_display_control  ( DeviceEventInfoEXT
                                                 , DisplayEventInfoEXT
                                                 , DisplayPowerInfoEXT
                                                 , SwapchainCounterCreateInfoEXT
                                                 ) where

import Vulkan.CStruct (FromCStruct)
import Vulkan.CStruct (ToCStruct)
import Data.Kind (Type)

data DeviceEventInfoEXT

instance ToCStruct DeviceEventInfoEXT
instance Show DeviceEventInfoEXT

instance FromCStruct DeviceEventInfoEXT


data DisplayEventInfoEXT

instance ToCStruct DisplayEventInfoEXT
instance Show DisplayEventInfoEXT

instance FromCStruct DisplayEventInfoEXT


data DisplayPowerInfoEXT

instance ToCStruct DisplayPowerInfoEXT
instance Show DisplayPowerInfoEXT

instance FromCStruct DisplayPowerInfoEXT


data SwapchainCounterCreateInfoEXT

instance ToCStruct SwapchainCounterCreateInfoEXT
instance Show SwapchainCounterCreateInfoEXT

instance FromCStruct SwapchainCounterCreateInfoEXT

